// Programming techniques
//
// Lecture 3 - Sorting algorithms
// Example
// ------------------------

#include <iostream>
#include <fstream>
#include <iomanip>
#include <cmath>

using namespace std;

//  Global Variables
double x[100], y[100];
double meanseg, mediseg, sdevseg;

//  Compute length of segment
double length(double x1, double y1, double x2, double y2)
{
    double s;
    s=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
    return s;
}

//  Data input (X,Y coordinates)
int inp_coords()
{
    int n=0;
    // Define input stream
    ifstream coord("points.txt", ios::in);
    if (! coord) {                               // Check validity of stream
        cerr<<"\n* Error opening input file *\n";
        exit (1); }
    else {                                       // Stream OK, proceed with input
        while (! coord.eof()) {                  // Read until end-of-file
            coord>>x[n]>>y[n];
            n++; }          
        coord.close();                           // Disconnect & close file
        }                       
    return n;    
}             

//  Compute array of segments
void segments(double arr[], int N)
{
     for (int k=1;k<N;k++)
         arr[k]=length(x[k-1],y[k-1],x[k],y[k]);
}

//  Write matrix elements to output file
void matrix_out(double arr[], int N)
{
    // Define output stream
    ofstream result("sortlen.txt", ios::out);
    if (! result) {                               // Check validity of stream
        cerr<<"\n* Error opening output file *\n";
        exit (1); }
    else {                                        // Stream OK, proceed with output
        result<<fixed<<setprecision(4);
        for (int k=1;k<N;k++)                     // Write all matrix elements
            result<<setw(16)<<arr[k]<<endl;
        result.close();                           // Disconnect & close file
        }                       
}    
 
//  Compute statistics of array
void stats(double arr[], int N)
{
     double mv=0;
     for (int k=1;k<N;k++)
         mv=mv+arr[k];
     meanseg=mv/N;
     mediseg=arr[N/2];
     mv=0;
     for (int k=1;k<N;k++)
         mv=mv+((arr[k]-meanseg)*(arr[k]-meanseg));
     sdevseg=sqrt(mv/(N-1));     
}
          
//  Write statistics to output file
void stat_out()
{
    // Define output stream
    ofstream result("sortlen.txt", ios::app);
    if (! result) {                               // Check validity of stream
        cerr<<"\n* Error opening output file *\n";
        exit (1); }
    else {                                        // Stream OK, proceed with output
        result<<fixed<<setprecision(4);
        result<<"Statistics of lengths\n";
        result<<"     Mean value = "<<setw(12)<<meanseg<<endl;
        result<<" Std. deviation = "<<setw(12)<<sdevseg<<endl;
        result<<"   Median value = "<<setw(12)<<mediseg<<endl;
        result.close();                           // Disconnect & close file        
        }
}

// Swap values of array elements
void swap(double arr[], int i, int j)
{
    double temp;
    temp=arr[i];
    arr[i]=arr[j];
    arr[j]=temp;
}

// Bubble sort of array
// --------------------
void bub_sort(double arr[], int N)
{
    int pass=1;
    bool sw=true;
    while (pass<N && sw) {
        sw=false;
        for (int i=0; i<N-pass; i++)
            if (arr[i]>arr[i+1]) {
                swap(arr,i,i+1);
                sw=true; }
        pass=pass+1; }
}

int main()
{
    int n;
    double s[100];

    n=inp_coords();
    segments(s,n);
    bub_sort(s,n);
    matrix_out(s,n);
    stats(s,n);
    stat_out();
    
    system("pause");
    return 0;
}
